/*  File BRM.H  Header file for Broom Balancing Dynamics and Graphics */

#ifndef BRM_H
#define BRM_H

/* Component index postions for vector representation of state variables */
#define X_POS_INDEX 1
#define X_DOT_INDEX 2
#define THETA_INDEX 3
#define THETA_DOT_INDEX 4
#define MAX_FORCE 10

extern const float
   x_pos_min,  /* End of cart path (meters) */
   x_pos_max,
   x_dot_min,
   x_dot_max,
   theta_min,  /* -pi/2 radians */
   theta_max,  /* pi/2 radians */
   theta_low_lim,  /* -12 degrees */
   theta_up_lim,   /*  12 degrees */
   theta_dot_min,
   theta_dot_max,
   force_mag; /* magnitude of force */

typedef struct { float
       x_pos,               /* position */
       x_dot,               /* velocity */
       theta,               /* angle */
       theta_dot;           /* angle velocity */
       } state_rec;

extern state_rec g_max_state,g_min_state;

extern float  x_pos_factor,x_dot_factor,theta_factor,theta_dot_factor;

typedef struct { float
      rand_force,
      system_force_incr,
      user_force_incr,
      zero_theta; } tbrm_setup_rec;

/* File BRSETVAR.C  Broom Game setup variables and constants  */

extern const float max_force;

extern tbrm_setup_rec  g_brm_setup;

void initialize_brm_setup (tbrm_setup_rec *the_setup_rec);

/* File BRMFNS.C  Miscellaneous Broom Balancing functions  */

state_rec x_to_state_rec (float *x,float x_min,float x_max);
void init_state_rec_factors (void);
void state_rec_to_x (state_rec the_state,float x_min,float x_max,float *x);

/* File BRMDYN.C   Broom dynamics  */

void new_broom_state (float frce,state_rec old_state,state_rec *new_state);
void force_effect (float force,state_rec in_state,state_rec *out_state);
void set_broom_at_start_position (state_rec *state);

/* File BRMGRCN.C  Screen constants for broom graphics  */
extern int window_bottom;
void set_pixel_positions (void);
void set_video_mode_and_colors (int video_driver,int *video_mode);

/* File BRMGRPH.C  Graphics for broom balancing  */
void write_brm_graph_screen (float x,float t,float f,int itr,short *page);

/* File BRMCNTRL.C Control System for broom balancing */
void control_system (char *title,tbrm_setup_rec br_set,
	   float (*get_control_force)(state_rec the_state));

/* File BRMGAME.C Manual Broom Balancing game */
void broom_game(tbrm_setup_rec brm_set);

#endif