/* File BRMCNTRL.C Control System for broom balancing */

#ifndef BRMCNTRL_C
#define BRMCNTRL_C

#include <dos.h>
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <graphics.h>
#include "ut.h"
#include "brm.h"

void control_system (char *title,tbrm_setup_rec br_set,
	   float (*get_control_force)(state_rec the_state));

/*  Implementation  */

void control_system (char *title,tbrm_setup_rec br_set,
	   float (*get_control_force)(state_rec the_state))
  {
   const float
   half_pi = 1.57,
   delay_ms = 100;  /* milliseconds of delay */

   state_rec the_state,new_state;
   float force;
   int graphdriver = DETECT,graphmode,errorcode;
   FILE *the_file;
   char ch,resp,ch1;
   int iters,out_freq = 5,out_iters = 0,out_limit = 200,out_start_iter = 0;
   float rand_start;
   short valid_key,page,quit;

do {
   clrscr ();
   textcolor (YELLOW);
   gotoxy (20,5);
   cprintf ("Welcome to %s !\r\n",title);
   gotoxy (1,8);
   cprintf ("Random starting force magnitude:           %5.2f\r\n",
	   br_set.rand_force);
   cprintf ("System controller force magnitude:         %5.2f\r\n",
	   br_set.system_force_incr);
   cprintf ("User input force magnitude:                %5.2f\r\n",
	   br_set.user_force_incr);
   cprintf ("Zero Angle Tolerance:                      %7.4f\r\n",
	   br_set.zero_theta);
   cprintf ("\r\n");
   cprintf ("(Use Special - Game Setup menu to change)\r\n");
   cprintf ("\r\n");
   cprintf ("Press key to start (esc to cancel, \r\n");
   cprintf ("      f to create output file BROOM.DAT): ");
   ch1 = getch();
   if (ch1 == ESC_KEY) return;
   if (ch1 == 'f') {
      cputs ("\r\nDefault values for iteration data collection: ");
      cprintf ("\r\nStarting Iteration: %d",out_start_iter);
      cprintf ("\r\nFrequency of Output Points: %d",out_freq);
      cprintf ("\r\nMax Output Points: %d\n",out_limit);
      cputs ("\r\n(Hit 'Enter' for default values)\n");
      cputs ("\r\nEnter new starting iteration: ");
      cget_int (6,&out_start_iter);
      cputs ("\r\nEnter new frequency: ");
      cget_int(4,&out_freq);
      cprintf ("\r\nEnter new max output points: ");
      cget_int(6,&out_limit);
      }
   initgraph (&graphdriver,&graphmode,"");
   errorcode = graphresult();
   if (errorcode != grOk) {
      cprintf ("\r\n");
      cprintf ("Graphics error (check BGI file).\r\n");
      cprintf ("Press key to exit: ");
      getch();
      exit (1);
      }
   set_video_mode_and_colors (graphdriver,&graphmode);
   set_pixel_positions();
   if (ch1 == 'f') the_file = fopen ("BROOM.DAT","wt");
   iters = 0;
   page = 0;
   set_broom_at_start_position (&the_state);
   write_brm_graph_screen (the_state.x_pos,the_state.theta,0,iters,&page);
   srand(1);
   rand_start = frand();
   if (rand_start > 0.5) force = br_set.rand_force;
   else force = -br_set.rand_force;
   delay (delay_ms);
   new_broom_state (force,the_state,&new_state);
   the_state = new_state;
   iters++;
   write_brm_graph_screen (the_state.x_pos,the_state.theta,force,iters,&page);
   quit = 0;
   if (ch1 == 'f') fprintf (the_file,"Iters  X Pos   Theta  Force\n");

   while ((the_state.x_pos > x_pos_min) &&
	 (the_state.x_pos < x_pos_max) &&
	 (the_state.theta > -half_pi) &&
	 (the_state.theta < half_pi) && (quit == 0)) {
      iters++;
      valid_key = 0;
      if (kbhit()) {
	 ch = getch();
	 if (ch == '\x0') {
	    ch = getch();
	    if (ch == LEFT_ARROW) {
	       valid_key = 1;
	       force = -br_set.user_force_incr;
	       }
	    if (ch == RIGHT_ARROW) {
	       valid_key = 1;
	       force = br_set.user_force_incr;
	       }
	    }
	 if (ch == ESC_KEY) quit = 1;
	 }
      if (valid_key == 0)
	 force = (*get_control_force) (the_state);
      new_broom_state (force,the_state,&new_state);
      the_state = new_state;
      write_brm_graph_screen (the_state.x_pos,the_state.theta,force,iters,&page);
      if (ch1 == 'f')
	 if (((iters == 1) || (iters % out_freq == 0))
	    && (iters >out_start_iter) && (out_iters <= out_limit)) {
	 fprintf (the_file,"%4d  %7.4f  %7.4f  %7.4f\n",iters,
	    the_state.x_pos,the_state.theta,force);
	 out_iters++;
	 }

      }  /* while */

   if (ch1 == 'f') fclose(the_file);

   settextjustify (LEFT_TEXT,TOP_TEXT);
   outtextxy (20,window_bottom + 40,"Game over!  Try again? y/n: ");

   do resp = getch();
   while ((resp != 'y') && (resp !='n'));

   closegraph();
   textmode(LASTMODE);

 } while (resp == 'y');

   textmode (LASTMODE);
   }

#endif
