/*  File FF.H  Header file for feedforward neural network  */

#ifndef FF_H
#define FF_H

#include "uttypes.h"
#include "ui.h"
#include "trn.h"

typedef struct {
      short allocated; /* TRUE if allocated, else FALSE  */
      f_vector x_inp,y_out;
      d_vector h1; /* Hidden layer nodes  */
      } network_rec;

typedef double **input_weights;
typedef double **output_weights;

typedef struct {
      short allocated; /* TRUE if weights allocated, else FALSE */
      input_weights wi;
      output_weights wo;
      } weights_rec;

typedef struct {
      float error_threshold;
      int start_iter;
      short shuffle;
      int no_of_inputs,no_of_h1_nodes,no_of_outputs;
      short in_weights_from_file;
      file_names_rec files;
      short sigmoid_out;
      float alpha,
	 learning_rate,
	 norm_in_min,
	 norm_in_range,
	 norm_out_min,
	 norm_out_range,
	 weight_range;
      } ff_setup_rec;

typedef struct {
     int high_water,last_right,last_wrong,right_count,wrong_count,
	 nbr_tr_items;
     long int iter;
     } display_rec; /* used for display during training */

typedef void  (*get_data_proc) (ff_setup_rec setup,training_rec tr_rec,
	   int item,f_vector x_in,f_vector desired_out);

typedef void (*training_proc) (ff_setup_rec setup,
	   weights_rec weights, training_rec tr_rec,
	   get_data_proc get_network_input_output);

/* FFSETUP.CPP  */
extern weights_rec g_system_weights;
extern ff_setup_rec g_ff_setup;

/*  FFWTS.C */
void move_2d_double (double **source,double **dest,int no_of_rows,
   int no_of_cols);
double **allocate_2d_double (int rows_upper_lim,int cols_upper_lim);
void free_2d_double (double **a);
weights_rec allocate_network_weights (ff_setup_rec setup);
void free_weights (weights_rec *weights);
short read_weights (weights_rec *weights,ff_setup_rec setup);
short write_weights (weights_rec weights,path_str file_name,
	 ff_setup_rec setup);
short print_weights (weights_rec weights,path_str file_name,
	 ff_setup_rec setup);
weights_rec initialize_weights (ff_setup_rec setup,short random_init);
#define RANDOM 1

/* FFEEDFW.C  */
void allocate_network (ff_setup_rec setup,network_rec *network);
void free_network (network_rec *network);
float ff_error (f_vector desired,f_vector net_out,ff_setup_rec setup);
void feedforward (network_rec net,weights_rec weights,
    ff_setup_rec setup);

/* FFBACKPR.C  */
void backpropagation_training (ff_setup_rec setup,weights_rec weights,
     training_rec tr_rec,get_data_proc get_network_input_output);

/* FFDISPL.C  */
void display_results (ff_setup_rec setup,display_rec display,
     f_vector d_out,f_vector net_out);
void setup_screen (ff_setup_rec setup,display_rec display);
void reset_screen (display_rec display);
int scale (float value);

/* FFTRAIN.C  */
void train_network_from_data (ff_setup_rec setup,
   training_proc the_training_proc);
void standard_data_input (ff_setup_rec setup,training_rec tr_rec,
   int item,f_vector x_in,f_vector desired_out);

/* FFRUN.C  */
void ff_network (ff_setup_rec setup,training_rec tr_rec,weights_rec weights,
     get_data_proc get_network_input_output);
void run_ff_network (ff_setup_rec setup);

#endif