// File FFINSETP.CPP Set up Turbo Vision interface for neural network
//        financial forecasting system.

#ifndef FFINSETP_CPP
#define FFINSETP_CPP

#include <string.h>

#include "ffsetup.cpp"

extern "C" {
   #include "ffin.h"
   }

void reset_tr_setup_enable (ffin_training_rec the_rec,
	  tlist_box_data *data_rec);
void tr_setup_processing (tlist_box_data *data_rec,int item_no);

ffin_training_rec  g_ffin_tr_setup;

/* Implementation  */

const min_year = 1953,
      max_year = 2050;

char input_files[MAX_NO_OF_INPUT_NODES][25] = {
    "Input Node #1 File Name",
    "Input Node #2 File Name",
    "Input Node #3 File Name",
    "Input Node #4 File Name",
    "Input Node #5 File Name",
    "Input Node #6 File Name",
    "Input Node #7 File Name",
    "Input Node #8 File Name",
    "Input Node #9 File Name",
    "Input Node #10 File Name" };

char output_files[MAX_NO_OF_OUTPUT_NODES][25] = {
    "Output Node #1 File Name",
    "Output Node #2 File Name",
    "Output Node #3 File Name",
    "Output Node #4 File Name",
    "Output Node #5 File Name"};

int set_file_names_start,
    set_input_nodes_start,
    set_input_nodes_end,
    set_output_nodes_start,
    set_output_nodes_end;

void training_rec_to_collection (ff_setup_rec *the_ff_setup,
   ffin_training_rec *the_rec,TDataCollection *the_collection)
   {
      void * value_ptr = NULL;
      tdata_type the_type;
      unsigned short len;
      boolean the_check = TRUE;
      unsigned short dec_places = 0;
      float the_max = 0.0;
      float the_min = 0.0;
      boolean enable = TRUE;
      int item_no = 0, i;


   value_ptr = &the_rec->no_of_data_points;
   the_type = INT_DATA;
   len = 4;
   dec_places = 0;
   the_max = MAX_NO_OF_INPUT_ITEMS - 1;
   the_min = 1;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "No. of Training Data Points",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));

   value_ptr = &the_rec->no_of_look_ahead_points;
   the_type = INT_DATA;
   len = 2;
   dec_places = 0;
   the_max = 10;
   the_min = 1;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "No. of Look Ahead Test Points",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));

   value_ptr = &the_ff_setup->files.training_file_name;
   the_type = PATH_DATA;
   len = MAX_ITEM_LEN;
   dec_places = 0;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "Output Training File Name",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"*.TRN",item_no));

   value_ptr = &the_ff_setup->files.test_file_name;
   the_type = PATH_DATA;
   len = MAX_ITEM_LEN;
   dec_places = 0;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "Output Test File Name",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"*.TST",item_no));

   value_ptr = &the_rec->no_of_input_nodes;
   the_type = INT_DATA;
   len = 5;
   dec_places = 0;
   the_max = MAX_NO_OF_INPUT_NODES;
   the_min = 1;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "No. of Input Nodes",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));

   set_input_nodes_start = item_no + 1;
   for (i=0;i<MAX_NO_OF_INPUT_NODES;i++) {
       the_type = PATH_DATA;
       len = MAX_ITEM_LEN;
       dec_places = 0;
       value_ptr = &the_rec->input_node_data[i].file_name;
       item_no++;
       the_collection->insert (new tcolor_data_obj (
	 input_files[i],
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"*.FDT",item_no));
       value_ptr = &the_rec->input_node_data[i].start.year;
       the_type = INT_DATA;
       len = 4;
       dec_places = 0;
       the_max = max_year;
       the_min = min_year;
       item_no++;
       the_collection->insert (new tcolor_data_obj (
	 "  Start Year",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));
       value_ptr = &the_rec->input_node_data[i].start.qtr;
       the_type = INT_DATA;
       len = 1;
       dec_places = 0;
       the_max = 4;
       the_min = 1;
       item_no++;
       the_collection->insert (new tcolor_data_obj (
	 "  Start Qtr",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));
	}  //  end i

   set_input_nodes_end = item_no;

   value_ptr = &the_rec->no_of_output_nodes;
   the_type = INT_DATA;
   len = 5;
   dec_places = 0;
   the_max = MAX_NO_OF_OUTPUT_NODES;
   the_min = 1;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "No. of Output Nodes",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));

   set_output_nodes_start = item_no + 1;
   for (i=0;i<MAX_NO_OF_OUTPUT_NODES;i++) {
       the_type = PATH_DATA;
       len = MAX_ITEM_LEN;
       dec_places = 0;
       value_ptr = &the_rec->output_node_data[i].file_name;
       item_no++;
       the_collection->insert (new tcolor_data_obj (
	 output_files[i],
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"*.FDT",item_no));
       value_ptr = &the_rec->output_node_data[i].start.year;
       the_type = INT_DATA;
       len = 4;
       dec_places = 0;
       the_max = max_year;
       the_min = min_year;
       item_no++;
       the_collection->insert (new tcolor_data_obj (
	 "  Start Year",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));
       value_ptr = &the_rec->output_node_data[i].start.qtr;
       the_type = INT_DATA;
       len = 1;
       dec_places = 0;
       the_max = 4;
       the_min = 1;
       item_no++;
       the_collection->insert (new tcolor_data_obj (
	 "  Start Qtr",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));
	}  //  end i

   set_output_nodes_end = item_no;

   return;
   } /* end proc */

class tffinsetapp : public tffsetapp
   {
   public :
     tffinsetapp();
     virtual void training_setup();
     static TMenuBar *initMenuBar (TRect r );
     static void init_submenus();
     virtual char *save_training_setup (char *the_file_name);
     virtual char *read_training_setup (char *the_file_name);
   };

tffinsetapp::tffinsetapp() : tffsetapp(),
   TProgInit (&tffinsetapp::initStatusLine,
	      &tffinsetapp::initMenuBar,
	      &tffinsetapp::initDeskTop
	      )
   {
   ((tcolor_data_obj *) (system_setup_data.item_collection)->
	 at(set_new_data_file_name - 1))->disable();
   training_rec_to_collection (&g_ff_setup,
	 &g_ffin_tr_setup,training_setup_data.item_collection);
   reset_tr_setup_enable (g_ffin_tr_setup,&training_setup_data);
   return;
   }

 void tffinsetapp::init_submenus()
   {
   tffsetapp::init_submenus();
   // Skip data and special submenus:
   train_submenu->next = NULL;
   return;
   }

TMenuBar *tffinsetapp::initMenuBar ( TRect r )
{
   r.b.y = r.a.y + 1;  //Set bottom line 1 line below top line
   init_submenus();
   return new TMenuBar (r,new TMenu(*about_submenu));
 }

void tffinsetapp::training_setup () {
     /* Setup dialog box for new data file information */
     reset_tr_setup_enable (g_ffin_tr_setup,&training_setup_data);
     list_dialog ("TRAINING PARAMETERS","",&training_setup_data,
	   MAX_NO_OF_SETUP_ITEMS,&tr_setup_processing);
     return;
     }  /* proc */

char *tffinsetapp::save_training_setup (char *the_file_name)
   {
   write_record (the_file_name,(void *)&g_ffin_tr_setup,
	 sizeof(ffin_training_rec));
   return the_file_name;
   }

char *tffinsetapp::read_training_setup (char *the_file_name)
   {
   read_record (the_file_name,(void *)&g_ffin_tr_setup,
	 sizeof(ffin_training_rec));
   return the_file_name;
   }

void reset_tr_setup_enable (ffin_training_rec the_rec,
	   tlist_box_data *the_data_rec) {
     short i;
     for (i = set_input_nodes_start;i<=set_input_nodes_end;i++)
	    ((tcolor_data_obj *) (the_data_rec->item_collection)->at(i - 1))
		 ->disable();
     for (i = set_output_nodes_start;i<=set_output_nodes_end;i++)
	    ((tcolor_data_obj *) (the_data_rec->item_collection)->at(i - 1))
		 ->disable();
     for (i = set_input_nodes_start;i<=
	 (set_input_nodes_start + 3*the_rec.no_of_input_nodes - 1);i++)
	    ((tcolor_data_obj *) (the_data_rec->item_collection)->at(i - 1))
		 ->enable();
     for (i = set_output_nodes_start;i<=
	 (set_output_nodes_start + 3*the_rec.no_of_output_nodes - 1);i++)
	    ((tcolor_data_obj *) (the_data_rec->item_collection)->at(i - 1))
		 ->enable();
     return;
     }  /* proc */

void tr_setup_processing (tlist_box_data *the_data_rec,int item_no) {
     ((tcolor_data_obj *) (the_data_rec->item_collection)->at(item_no - 1))
	  ->get_new_value();
     reset_tr_setup_enable (g_ffin_tr_setup,the_data_rec);
     return;
     }  /* end proc */

#endif