/*  File FLPRCS.C  Fuzzy Logic procedures and functions  */

#ifndef FLPRCS_C
#define FLPRCS_C

#define TOO_SMALL 1e-6

#include <stdlib.h>
#include <math.h>
#include <conio.h>

#include "flogic.h"

fuzzy_system_rec g_fuzzy_system;

trapezoid init_trapz (float x1,float x2,float x3,float x4,
   trapz_type typ);
float fuzzy_system (float inputs[],fuzzy_system_rec fl);
void free_fuzzy_rules (fuzzy_system_rec *fz);

/*  Implementation  */

trapezoid init_trapz (float x1,float x2,float x3,float x4,
   trapz_type typ) {
   trapezoid trz;
   trz.a = x1;
   trz.b = x2;
   trz.c = x3;
   trz.d = x4;
   trz.tp = typ;
   switch (trz.tp) {
      case regular:
	 trz.l_slope = 1.0/(trz.b - trz.a);
	 trz.r_slope = 1.0/(trz.c - trz.d);
	 break;
      case left:
	 trz.r_slope = 1.0/(trz.a - trz.b);
	 trz.l_slope = 0.0;
	 break;
      case right:
	 trz.l_slope = 1.0/(trz.b - trz.a);
	 trz.r_slope = 0.0;
	 break;
      }  /* end switch  */
   return trz;
   }  /* end function */

float trapz (float x, trapezoid trz) {
   switch (trz.tp) {
      case left:
	 if (x <= trz.a)
	    return 1.0;
	 if (x >= trz.b)
	    return 0.0;
	 /* a < x < b */
	 return trz.r_slope * (x - trz.b);
      case right:
	 if (x <= trz.a)
	    return 0.0;
	 if (x >= trz.b)
	    return 1.0;
	 /* a < x < b */
	 return trz.l_slope * (x - trz.a);
      case regular:
	 if ((x <= trz.a) || (x >= trz.d))
	    return 0.0;
	 if ((x >= trz.b) && (x <= trz.c))
	    return 1.0;
	 if ((x >= trz.a) && (x <= trz.b))
	    return trz.l_slope * (x - trz.a);
	 if ((x >= trz.c) && (x <= trz.d))
	    return  trz.r_slope * (x - trz.d);
	 }  /* End switch  */
   return 0.0;  /* should not get to this point */
   }  /* End function */

float min_of (float values[],int no_of_inps) {
   int i;
   float val;
   val = values [0];
   for (i = 1;i < no_of_inps;i++) {
       if (values[i] < val)
	  val = values [i];
   }
   return val;
   }

float fuzzy_system (float inputs[],fuzzy_system_rec fz) {
   int i,j;
   short variable_index,fuzzy_set;
   float sum1 = 0.0,sum2 = 0.0,weight;
   float m_values[MAX_NO_OF_INPUTS];
   for (i = 0;i < fz.no_of_rules;i++) {
       for (j = 0;j < fz.no_of_inputs;j++) {
	   variable_index = fz.rules[i].inp_index[j];
	   fuzzy_set = fz.rules[i].inp_fuzzy_set[j];
	   m_values[j] = trapz(inputs[variable_index],
	       fz.inp_mem_fns[variable_index][fuzzy_set]);
	   } /* end j  */
       weight = min_of (m_values,fz.no_of_inputs);
       sum1 += weight * fz.output_values[fz.rules[i].out_fuzzy_set];
       sum2 += weight;
       } /* end i  */
   if (fabs(sum2) < TOO_SMALL) {
      cprintf ("\r\nFLPRCS Error: Sum2 in fuzzy_system is 0.  Press key: ");
      getch();
      exit(1);
      return 0.0;
      }
   return (sum1/sum2);
   }  /* end fuzzy_system  */

void free_fuzzy_rules (fuzzy_system_rec *fz) {
     if (fz->allocated)
	free (fz->rules);
     fz->allocated = FALSE;
     return;
     }

#endif